﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Linq;
using System.Threading;
using JoeScan.JCamNet;

namespace Examples
{
    class Program
    {
        static void Main(string[] args)
        {
            IPAddress baseIpAddress = new IPAddress(new byte[] { 192, 168, 1, 151 });
            Scanner scanner = null;
            try
            {
                scanner = Scanner.Connect(baseIpAddress, 0);
            }
            catch (Exception e)
            {
                Console.WriteLine("Failed to connect to scanner: {0}, Reason: {1}",
                    baseIpAddress.ToString(), e.Message);
                Console.ReadLine();
                return;
            }
            TextReader parametersReader;
            using (parametersReader = File.OpenText("param.dat"))
            {
                string parametersString = parametersReader.ReadToEnd();
                try
                {
                    scanner.SetParameters(parametersString, true);
                }
                catch (Exception e)
                {
                    Console.WriteLine("Scanner {0}:{1} did not accept configuration parameters!",
                        scanner.IPAddress.ToString(), scanner.CableID);
                    Console.ReadLine();
                    return;
                }
            }

            try
            {
                for (int i = 0; i < 5; i++)
                {
                    Console.WriteLine();
                    RawScan scan;

                    int cameraExposure = 100 + (i * 10000);
                    int laserExposure = 0;

                    scan = scanner.GetDiagnosticScan(cameraExposure, laserExposure);

                    // Report the number of points that the camera detected
                    var pointsFound = scan.Sum(pt => pt.Status != DataPointStatus.NoData ? 1 : 0);
                    Console.WriteLine("  Found " + pointsFound + " lit points with camera exposure {0}, laser exposure {1}", cameraExposure, laserExposure);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error retrieving raw scan data from scanner. Exiting.");
                Console.ReadLine();
                return;
            }

            Console.WriteLine("\nPress [Enter] to exit the application.");
            Console.ReadLine();
        }

    }
}
